from TDhelper.network.http.REST_HTTP import POST, GET
from datetime import datetime, timedelta, timezone
import json


class token_manage:
    __token = None
    __expire = None
    __token_service_uri = None
    __service_key = None
    __service_secret = None
    __time_cls = None
    __status= False
    __err_msg= ""
    
    @property
    def Status(self):
        return self.__status
    
    @property
    def ErrMsg(self):
        return self.__err_msg
    
    @property
    def Token(self):
        if self.__status:
            try:
                if not self.__token:
                    self.__get_token__()
                if self.__expire:
                    if datetime.strptime(self.__expire,"%Y-%m-%d %H:%M:%S.%f") < self.__time_cls.now():
                        self.__get_token__()
                return self.__token
            except Exception as e:
                self.__status= False
                self.__err_msg= e.args[0]
        else:
            return self.__err_msg

    def __init__(self, token_uri:str, svr_key:str, secret:str, time_cls=None, path:str= None, post_data:dict={}) -> None:
        self.__time_cls = datetime if not time_cls else time_cls
        self.__service_key = svr_key
        self.__service_secret = secret
        token_uri= token_uri.rstrip('/')
        self.__token_service_uri = "/".join([token_uri, path if path else "api/rpc_token"])
        self.__post_data=post_data
        if "svr_key" not in self.__post_data:
            self.__post_data.update({"svr_key":self.__service_key})
        if "secret" not in self.__post_data:
            self.__post_data.update({"secret":self.__service_secret})
        self.__status= False
        self.__get_token__()

    def __get_token__(self):
        try:
            state, body = GET(
                self.__token_service_uri,
                self.__post_data,
                time_out=5
            )
            if state:
                data = json.loads(body)
                if data.get("state")==200:
                    self.__token = data.get("msg").get("token", None)
                    self.__expire = data.get("msg").get("expire", None).replace("T"," ")
                    self.__status= True
                else:
                    self.__status= False
                    self.__err_msg= data.get('msg')
            else:
                self.__status= False
                self.__err_msg= data.get('msg')
        except Exception as e:
            self.__status= False
            self.__err_msg= e.args[0]



from model import model, createCls
from FieldsType import FieldType


class categories(model):
    category_id = FieldType(int, "type_id")
    category_name = FieldType(str, "type_name")

    def __init__(self, source, convert_type="json"):
        super(categories, self).__init__(source, convert_type)


if __name__ == "__main__":
    api_json = '[{"type_id": 1, "type_name": "电影"},{"type_id": 2, "type_name": "电影2"},{"type_id": 3, "type_name": "电影3"}]'#[{"type_id": 1, "type_name": "电影"},{"type_id": 2, "type_name": "电影2"},{"type_id": 3, "type_name": "电影3"}] #{"type_id": 1, "type_name": "电影"}#
    mapping = {"type_id": [int], "name": [str,"type_name"]}
    a = createCls("categories", mapping, api_json) #categories(api_json)
    for o in a.items():
        print(o.type_id)
        print(o.name)
        #print(o.category_id)
        #print(o.category_name)



import logging
import time


class performance:
    __state__ = True

    @classmethod
    def on_off(self, on=True):
        self.__state__ = on

    @classmethod
    def performance_testing(self, func):
        def wapper(*args, **kwargs):
            if self.__state__:
                start = time.clock_gettime(0)
                result = func(*args, **kwargs)
                end = time.clock_gettime(0)
                logging.info("execute function %s consuming time %f(s)." % (
                    func.__name__, end-start))
                return result
            else:
                return func(*args, **kwargs)
        return wapper



#!/usr/bin/env python3.6
# -*- encoding: utf-8 -*-
'''
@File    :   m3u8b.py
@Time    :   2020/04/23 12:54:28
@Author  :   Tang Jing 
@Version :   1.0.0
@Contact :   yeihizhi@163.com
@License :   (C)Copyright 2020
@Desc    :   None
'''

# here put the import lib
import re
import os
import time
# from Crypto.Cipher import AES 解密后CKPLAYER播放有问题.
from decimal import *
from threading import Thread
from threading import Lock
from threading import Event as ThreadEvent
from urllib.parse import urlparse
from io import StringIO, BytesIO
from TDhelper.cache.pools import pools
from TDhelper.network.http.http_helper import m_http
from TDhelper.network.http.status.M3U8_STATUS import M3U8_STATUS

# code start
cfg = {
    "cfg": {
        'timeout': 5,  # 超时(秒)
        'reconnect': 5  # 连接重试次数
    },
    "save-folder": "tmp\\ts",  # 临时文件件
    "download-poolsize": 20,  # 下载线程
    "buffer-index": 5  # 下载多少文件后生成索引文件
}


class m3u8:
    def __init__(self, m3u8Cfg: dict = None):
        self.setting = m3u8Cfg if m3u8Cfg else cfg  # 如果不传配置参数，则使用默认参数.
        self._state = True  # 运行状态
        self._http = m_http()
        # requests 连接失败重连次数
        self._reconnect = self.setting['cfg']['reconnect']
        # requests超时时间
        self._timeout = self.setting['cfg']['timeout']
        # 文件保存文件夹
        self._save_folder = self.setting['save-folder']
        self._m3u8_file_hash = None
        self._create_m3u8_index_file = False  # 是否已经建立M3U8播放索引
        self._del_state = False  # 是否正在删除过期文件,LIVE模式需要使用.
        # 密钥
        self._m3u8_key = None
        # 下载文件线程池配置
        self._lock__ = Lock()  # 线程锁
        self._download_event__ = ThreadEvent()  # 下载线程同步
        self._complete_event__ = ThreadEvent()  # 下载完成通知事件
        self._event_create_m3u8_file__ = ThreadEvent()  # 生成m3u8索引文件事件
        self._download_complate_total__ = 0  # 统计下载完成数量
        self._download_thread_active_ = 0  # 下载线程活跃数,用于判断文件是否下载完成.
        self._downlaod_buffer_length = 0  # 下载缓存长度, 用于计算快偏移量
        self._download_pool_size = self.setting['download-poolsize']
        self._download_pool = pools(self._download_pool_size)

        # 初始化下载线程池
        for i in range(0, self._download_pool_size):
            self._download_pool.push(m_http())
        self._save_folder = self.__checkfolder__(self._save_folder)

        # 下载缓存索引，下载索引到达这个值时才生成播放索引文件
        self._ts_save_folder = None
        self._m3u8_buffer_index = self.setting['buffer-index']
        self._current_m3u8_file = None
        # live:直播(需要连续多次下载M3U8文件),whole:完整播放(点播不需要多次下载M3U8文件)
        self._play_mode__ = 'whole'
        # M3U8 TAGS
        self._prefix_url = None
        self._EXT_X_TARGETDURATION__ = None
        self._EXTINF__ = []
        self._EXT_X_VERSION__ = None
        self._EXT_X_BYTERANGE__ = None
        self._EXT_X_KEY__ = None
        self._EXT_X_MAP__ = None
        self._EXT_X_PROGRAM_DATE_TIME__ = None
        self._EXT_X_DATERANGE__ = None
        self._EXT_X_MEDIA_SEQUENCE__ = None
        self._EXT_X_DISCONTINUITY_SEQUENCE__ = None
        self._EXT_X_PLAYLIST_TYPE__ = None
        self._EXT_X_MEDIA__ = None
        self._EXT_X_STREAM_INF__ = None
        self._EXT_X_I_FRAME_STREAM_INF__ = None
        self._EXT_X_SESSION_DATA__ = None
        self._EXT_X_SESSION_KEY__ = None
        self._EXT_X_START__ = None
        self._EXT_X_DISCONTINUITY__ = None
        self._EXT_X_ENDLIST__ = None
        self._EXT_X_I_FRAMES_ONLY__ = None
        self._EXT_X_INDEPENDENT_SEGMENTS__ = None

        # 自己定义M3U8 处理属性
        self._EXT_X_M3U8_FILE_LIST__ = []  # 直播类有m3u8有多个m3u8文件地址
        self._EXT_X_M3U8_PLAY_LIST__ = []  # m3u8播放列表TS文件列表
        self._EXT_X_M3U8_PLAY_LIST_INDEX__ = 0  # 下载m3u8播放列表TS文件列表索引
        self._LIVE_DOWNLOAD_FILE = []  # 直播下载文件

    def __resetVar__(self):
        '''
            重置变量
        '''
        self._ts_save_folder = None
        self._EXT_X_KEY__ = None
        self._download_event__.clear()
        self._complete_event__.clear()
        self._event_create_m3u8_file__.clear()
        self._download_complate_total__ = 0
        self._create_m3u8_index_file = False
        self._EXT_X_M3U8_FILE_LIST__ = []
        self._EXT_X_M3U8_PLAY_LIST__ = []
        self._EXT_X_M3U8_PLAY_LIST_INDEX__ = 0
        self._EXTINF__ = []
        self._download_complate_total__ = 0
        self._download_thread_active_ = 0
        self._downlaod_buffer_length = 0
        self._del_state = False
        if self._play_mode__ == "whole":
            self._LIVE_DOWNLOAD_FILE = []
            self._m3u8_file_hash = None

    def Get(self, m3u8_url, ts_save_path: str = None, download_start_offset: int = 0):
        try:
            self._EXT_X_M3U8_PLAY_LIST_INDEX__ = download_start_offset
            m_count = 0
            while True:
                if not self._state:
                    self._event_create_m3u8_file__.set()
                    self._download_event__.set()
                    self._complete_event__.set()
                    break
                self.__resetVar__()
                if ts_save_path:
                    self._ts_save_folder = ts_save_path
                else:
                    return 'ts save folder is none.', M3U8_STATUS.ERROR
                ret, status = self.__getM3U8file__(m3u8_url)
                if status == M3U8_STATUS.WAIT:
                    time.sleep(1)
                elif status == M3U8_STATUS.SUCCESS:
                    if self._play_mode__ == 'live':
                        self._ts_save_folder = 'live'
                        if m_count == 0:
                            m_count += 1
                            if os.path.exists(os.path.join(self._save_folder, self._ts_save_folder)):
                                for item in os.listdir(os.path.join(self._save_folder, self._ts_save_folder)):
                                    os.remove(os.path.join(self._save_folder, *(self._ts_save_folder,item))) #直播类，删除已下载TS文件.
                    self.__get_key__()  # 下载KEY文件.模块内有判断是否需要下载.
                    ret, status = self.__download_ts__()
                    if self._play_mode__ == 'live':
                        continue
                    else:
                        if status == M3U8_STATUS.SUCCESS:
                            return 'SUCCESS', M3U8_STATUS.SUCCESS
                        return ret, status
                elif status == M3U8_STATUS.ERROR:
                    return ret, M3U8_STATUS.ERROR
                else:
                    return 'Unknow error.', M3U8_STATUS.ERROR
        finally:
            self._event_create_m3u8_file__.set()
            self._complete_event__.set()
            self._download_event__.set()

    def getComplate(self, func):
        self._complete_event__.wait()
        func((), {})
        self._complete_event__.clear()

    def GetM3U8File(self):
        '''
            获取生成的M3U8索引文件
        '''
        self._event_create_m3u8_file__.wait()  # 阻塞等待生成播放索引.
        self._event_create_m3u8_file__.clear()  # 重置状态.
        return os.path.join(self._save_folder, *(self._ts_save_folder, 'index.m3u8',)), M3U8_STATUS.SUCCESS

    def close(self):
        self._state = False
        self._event_create_m3u8_file__.set()
        self._complete_event__.set()
        self._download_event__.set()

    def seek(self, time):
        ''''
        寻道时间定位
        '''
        total_time = 0
        index = 0
        for item in self._EXT_X_M3U8_PLAY_LIST__:
            total_time += float(item['duration'])
            if total_time >= time:
                self._EXT_X_M3U8_PLAY_LIST_INDEX__ = index
                break
            index += 1

    def __checkfolder__(self, folder):
        # 检查临时文件夹路径是否完整
        self._lock__.acquire()
        try:
            if not re.search(r':\\', folder, re.I | re.M):
                folder = os.path.join(os.getcwd(), folder)
            # 如果文件夹不存在则建立文件夹
            if not os.path.exists(folder):
                os.mkdir(folder)
        except OSError as e:
            raise e
        self._lock__.release()
        return folder

    def __getM3U8file__(self, uri: str):
        '''
            下载m3u8文件
        '''
        self._prefix_url = uri.rsplit('/', 1)[0]  # 获取URI前缀
        self._current_m3u8_file = uri  # 当前下载M3U8 URI
        print('download:%s'% uri)
        response, status = self.__request__(uri)
        if status == 200:
            print('download:%s Ok'% uri)
            if response == self._m3u8_file_hash:
                return uri, M3U8_STATUS.WAIT  # m3u8文件没有变化.返回等待状态.
            self._m3u8_file_hash = response
            self.__analysis__(response)
            if self._EXT_X_STREAM_INF__:
                self._EXT_X_STREAM_INF__ = None  # 重置STREAM_INF属性
                # 下载下一层M3U8.
                return self.__getM3U8file__(self.__spell_uri__(
                    self._EXT_X_M3U8_FILE_LIST__[0]))
            else:
                if self._EXT_X_ENDLIST__:
                    self._play_mode__ = 'whole'
                    self._m3u8_buffer_index = self.setting['buffer-index']
                else:
                    self._play_mode__ = 'live'
                    self._m3u8_buffer_index = 1
                return uri, M3U8_STATUS.SUCCESS
        else:
            print('download:%s Error'% uri)
            self._complete_event__.set()
            self._event_create_m3u8_file__.set()
            return uri, M3U8_STATUS.ERROR

    def __download_ts__(self):
        if self._EXTINF__:
            # 生成ts文件下载列表.
            m_index = 0
            for item in self._EXTINF__:
                m_file = item.split(',')[1]
                duration = item.split(',')[0]
                if not re.match(r"^(http|https)://", m_file, re.I | re.M):
                    download_uri = self.__spell_uri__(m_file)
                else:
                    download_uri = m_file
                local_file_name = m_file.rsplit('/', 1)
                if len(local_file_name) == 1:
                    local_file_name = local_file_name[0]
                else:
                    local_file_name = local_file_name[1]
                m_download_struct = {'index': m_index, 'url': download_uri, 'offset': None,
                                     'duration': duration, 'filename': local_file_name, 'state': False}
                self._EXT_X_M3U8_PLAY_LIST__.append(m_download_struct)
                m_index += 1
            # 开始下载
            for i in range(0, self._download_pool_size):
                Thread(target=self.__get_ts_thread__).start()
            self._download_event__.wait()  # 阻塞，等待下载线程完成下载
            self._download_event__.clear()
            self._complete_event__.set()
            return 'SUCCESS', M3U8_STATUS.SUCCESS
        else:
            return 'EXTINF IS NONE.', -1

    def __get_ts_thread__(self):
        '''
        TS文件下载
        '''
        self._lock__.acquire()
        if self._download_thread_active_ < self._download_pool_size:
            self._download_thread_active_ += 1
        else:
            raise Exception('download thread error: thread active total count({0}) Over length ({1}).'.format(
                self._download_thread_active_, self._download_pool_size))
        # thiread_id = self._download_thread_active_
        self._lock__.release()
        m_http_control = None
        m_http_control = self._download_pool.pop()
        m_index = 0
        if m_http_control:
            while True:
                if not self._state:
                    # 调用close方法，退出下载线程
                    self._download_pool.push(m_http_control)
                    break
                ts_struct = None  # TS文件下载信息结构体
                # 从播放列表取TS文件地址
                if self._EXT_X_M3U8_PLAY_LIST_INDEX__ < len(self._EXT_X_M3U8_PLAY_LIST__):
                    ts_struct = self._EXT_X_M3U8_PLAY_LIST__[
                        self._EXT_X_M3U8_PLAY_LIST_INDEX__]
                    m_index = self._EXT_X_M3U8_PLAY_LIST_INDEX__
                    self._lock__.acquire()
                    self._EXT_X_M3U8_PLAY_LIST_INDEX__ += 1
                    self._lock__.release()
                else:
                    # 下载完毕跳出下载线程
                    self._download_pool.push(m_http_control)
                    break
                if ts_struct:
                    if ts_struct['state']:
                        continue
                    if os.path.exists(os.path.join(self._save_folder, *(self._ts_save_folder, ts_struct['filename']))):
                        continue
                    file_bytes, status = m_http_control.download(
                        ts_struct['url'])
                    if status != 200:
                        i = 0
                        while True:
                            file_bytes, status = m_http_control.download(
                                ts_struct['url'])
                            i += 1
                            if i > self._reconnect or status == 200:  # if status==200:
                                break
                    if status == 200:
                        total_download_rate=self._EXT_X_M3U8_PLAY_LIST_INDEX__/len(
                            self._EXT_X_M3U8_PLAY_LIST__)*100
                        if total_download_rate>100.00:
                            total_download_rate= 100.00
                        total_download_rate=Decimal(str(total_download_rate)).quantize(Decimal('0.00'))
                        print('\rdownload:{1}({0}%)'.format(total_download_rate, self._current_m3u8_file), end='')
                        # 建立播放索引.
                        if ts_struct['index'] >= self._m3u8_buffer_index:
                            self.__create_m3u8_play_file__()
                        self._lock__.acquire()
                        ts_struct['state']=True # 更改TS下载列表对应文件下载状态.#self._EXT_X_M3U8_PLAY_LIST__[m_index]['state'] = True
                        ts_struct['offset'] = (
                            self._downlaod_buffer_length, len(file_bytes))
                        self._downlaod_buffer_length += len(file_bytes)
                        self._EXT_X_M3U8_PLAY_LIST__[
                            ts_struct['index']] = ts_struct
                        self._lock__.release()
                        # 用索引号替代原TS文件名称 ts_struct['url'].rsplit('/', 1)[1]
                        tmp_file_name = str(
                            ts_struct['filename']).split('?')[0]
                        # 直播类过期TS文件删除，模块内判断是否是直播.
                        self.__del_live_ts__(tmp_file_name)
                        '''if self._m3u8_key:
                        # 解密视频文件(CKPlayer播放器解密后的TS不能连续下载)不知道原因
                        cryptor = AES.new(bytes(self._m3u8_key['key'], encoding='utf-8'), AES.MODE_CBC, bytes(
                            self._m3u8_key['key'], encoding='utf-8'))
                        file_bytes = cryptor.decrypt(file_bytes)'''
                        # 生成TS文件.
                        m_saveTsPath = os.path.join(
                            self._save_folder, self._ts_save_folder)
                        self.__checkfolder__(m_saveTsPath)
                        with open(os.path.join(m_saveTsPath, tmp_file_name), mode="wb") as tmp_stream:
                            tmp_stream.write(file_bytes)
                            tmp_stream.flush()
                            tmp_stream.close()

                else:
                    continue  # 下载信息为空则进入下个下载循环.
            # 下载完成退出逻辑.
            self._lock__.acquire()
            self._download_thread_active_ -= 1
            event_state = False
            if self._download_thread_active_ == 0:
                event_state = True
            if event_state:
                self._download_event__.set()
            self._lock__.release()
        else:
            raise Exception('httpContent is none.')

    def __del_live_ts__(self, ts_file_name):
        '''
            删除直播过期文件.
        '''
        self._lock__.acquire()
        if self._del_state:
            return None
        self._del_state = True
        if self._play_mode__ == 'live':
            if len(self._LIVE_DOWNLOAD_FILE) < len(self._EXTINF__)*4:
                self._LIVE_DOWNLOAD_FILE.append(ts_file_name)
            else:
                count = 0
                size = int((len(self._EXTINF__)*4)/2)
                while True:
                    if count < size:
                        try:
                            os.remove(os.path.join(
                                self._save_folder, *(self._ts_save_folder, self._LIVE_DOWNLOAD_FILE[0])))
                            del self._LIVE_DOWNLOAD_FILE[0]
                        except Exception as e:
                            raise e
                    else:
                        break
                    count += 1
                self._LIVE_DOWNLOAD_FILE.append(ts_file_name)
        self._del_state = False
        self._lock__.release()

    def __create_m3u8_play_file__(self):
        '''
            建立.m3u8播放索引文件
        '''
        if self._create_m3u8_index_file:
            return None
        tmp_saveTsPath = os.path.join(self._save_folder, self._ts_save_folder)
        self.__checkfolder__(tmp_saveTsPath)
        with open(os.path.join(tmp_saveTsPath, 'index.m3u8'), mode='wb') as f:
            f.writelines([bytes('#EXTM3U\n', encoding='utf-8')])
            f.writelines([bytes('#EXT-X-ALLOW-CACHE:YES\n', encoding='utf-8')])
            for item in self.__dict__:
                if self.__dict__[item]:
                    if re.match('_EXT_X_VERSION', item, re.I | re.M):
                        if self._EXT_X_VERSION__:
                            m_line = "#EXT-X-VERSION:"+self._EXT_X_VERSION__+"\n"
                            f.writelines([bytes(m_line, encoding='utf-8')])
                    elif re.match('_EXT_X_KEY__', item, re.I | re.M):
                        m_line = "#EXT-X-KEY:METHOD={0},URI=\"key.key\"\n".format(
                            self._m3u8_key['method'])
                        f.writelines([bytes(m_line, encoding='utf-8')])
                    elif re.match('_EXT_X_MEDIA_SEQUENCE', item, re.I | re.M):
                        m_line = ''
                        if self._EXT_X_MEDIA_SEQUENCE__:
                            m_line = "#EXT-X-MEDIA-SEQUENCE:"+self._EXT_X_MEDIA_SEQUENCE__+"\n"
                        else:
                            m_line = "#EXT-X-MEDIA-SEQUENCE:0\n"
                        f.writelines([bytes(m_line, encoding='utf-8')])
                    elif re.match('_EXT_X_ALLOW_CACHE', item, re.I | re.M):
                        pass
                    elif re.match('_EXT_X_TARGETDURATION', item, re.I | re.M):
                        if self._EXT_X_TARGETDURATION__:
                            m_line = '#EXT-X-TARGETDURATION:'+self._EXT_X_TARGETDURATION__+"\n"
                            f.writelines([bytes(m_line, encoding='utf-8')])
                    elif re.match('_EXT_X_M3U8_PLAY_LIST__', item, re.I | re.M):
                        for extinf in self._EXT_X_M3U8_PLAY_LIST__:
                            m_line = '#EXTINF:' + \
                                extinf['duration']+',\n' + \
                                str(extinf['filename'].split('?')[0])+"\n"
                            f.writelines([bytes(m_line, encoding='utf-8')])
                        if self._EXT_X_ENDLIST__:
                            m_line = '#EXT-X-ENDLIST\n'
                            f.writelines([bytes(m_line, encoding='utf-8')])
            f.flush()
            f.close()
        self._create_m3u8_index_file = True
        self._event_create_m3u8_file__.set()  # 解锁M3U8文件生成成功锁

    def __get_key__(self):
        '''
            获取密钥文件
        '''
        if self._EXT_X_KEY__:
            with StringIO(self._EXT_X_KEY__.replace(',', "\r\n")) as f:
                self._m3u8_key = {'method': None, 'key': '', 'iv': None}
                while True:
                    line = f.readline()
                    if line:
                        line = re.sub('\"|\'|\\r|\\n', '',
                                      line, count=0, flags=0)
                        line = line.split('=')
                        if len(line) == 2:
                            if line[0].lower() == 'method':
                                self._m3u8_key['method'] = line[1]
                            elif line[0].lower() == 'uri':
                                key_url = ''
                                if not re.match("http://|https://", line[1], re.I | re.M):
                                    key_url = self.__spell_uri__(line[1])
                                    state = False
                                    content = ''
                                    download_count = 0
                                    while True:
                                        content, state = self.__request__(
                                            key_url)
                                        download_count += 1
                                        if download_count > self._reconnect:
                                            break
                                        if state == 200:
                                            self._m3u8_key['key'] = content
                                            self.__checkfolder__(os.path.join(self._save_folder,self._ts_save_folder))
                                            with open(os.path.join(self._save_folder, *(self._ts_save_folder, 'key.key')), mode='wb') as keyStream:
                                                keyStream.write(
                                                    bytes(content, encoding='utf-8'))
                                                keyStream.flush()
                                            break
                            elif line[0].lower() == 'iv':
                                self._m3u8_key['iv'] = line[1]
                    else:
                        break
        else:
            return None

    def __spell_uri__(self, uri):
        '''
            拼接资源URI
        '''
        ret= ''
        if uri[0]=='/':
            if re.search(r'http:',self._prefix_url, re.M|re.I):
                m_offset= self._prefix_url.find('/',7)
                if m_offset>0:
                    ret= self._prefix_url[0:m_offset]
            elif re.search(r'https:',self._prefix_url, re.M|re.I):
                m_offset= self._prefix_url.find('/',8)
                if m_offset>0:
                    ret= self._prefix_url[0:m_offset]
            else:
                ret= self._prefix_url.rsplit('/',1)[0]
        else:
            ret = self._prefix_url
        for item in uri.split('/'):
            if not re.search(item, self._prefix_url, re.M | re.I):
                ret += '/'+item
        return ret

    def __analysis__(self, content):
        '''
            分析M3U8文件
        '''
        m_result = self.__match__(r"^#EXTM3U", content)
        if m_result:
            content = re.sub(r",(\r\n|\r|\n)", ",", content, count=0, flags=0)
            for line in StringIO(content):
                self.__m3u8_attr__(line)
        else:
            return '', M3U8_STATUS.TYPE_ERROR

    def __m3u8_attr__(self, content):
        '''
            设置M3U8 TAG值
        '''
        content = re.sub(r"^\r\n|\r|\n", "", content, count=0, flags=0)
        m_result = re.match(r"^#.*", content)
        if m_result:
            m_data = content.split(":", 1)
            m_key = None
            if len(m_data) == 2:
                m_key = m_data[0]
            else:
                m_key = content
            m_key += "__"
            m_key = re.sub(r"#|-", "_", m_key, count=0, flags=0)
            m_key = m_key.replace(" ", "")
            if len(m_data) == 2:
                if m_key == "_EXTINF__":
                    self._EXTINF__.append(m_data[1])
                else:
                    self.__setattr__(m_key, m_data[1])
            else:
                self.__setattr__(m_key, True)
        else:
            m_stream= StringIO(content)
            m_line= m_stream.readline()
            while m_line:
                if re.search(r'/|.m3u8', content):
                    self._EXT_X_M3U8_FILE_LIST__.append(m_line)
                m_line= m_stream.readline()
            m_stream= None

    def __match__(self, regex, content):
        '''
            验证字符串是否存在
        '''
        m_result = re.match(regex, content, re.I | re.M)
        if m_result:
            return True
        return False

    def __request__(self, url):
        '''
            requests
        '''
        if self._http:
            try:
                for i in range(0, self._reconnect+1):
                    response, status = self._http.getcontent(
                        url, p_timeout=self._timeout)
                    if status == 200:
                        return response, status
                if i+1 >= self._reconnect:
                    return 'timeout', 408
            except Exception as e:
                return e, -1
        else:
            return 'http is none.', -1



import datetime
from enum import Enum

from TDhelper.Spider.models.Cache_L2_Model import objectId

class fingerprint(objectId):
    def __init__(self):
        super(fingerprint, self).__init__()
        self.model['fingerprint']= None
        self.model['url']= None
        self.model['lastupdatetime']= datetime.datetime.now()

    @property
    def Fingerprint(self):
        return self.model['fingerprint']

    @Fingerprint.setter
    def Fingerprint(self, value):
        self.model['fingerprint']= value

    @property
    def Url(self):
        return self.model['url']

    @Url.setter
    def Url(self, value):
        self.model['url']= value

    @property
    def LastUpdateTime(self):
        return self.model['lastupdatetime']

    @LastUpdateTime.setter
    def LastUpdateTime(self, value):
        self.model['lastupdatetime']= value


import logging
import json
import time
from TDhelper.network.http.REST_HTTP import GET
from TDhelper.generic.dynamic.base.Meta import dynamic_creator
from TDhelper.network.rpc.Core.struct import RPC_SERVICE
from TDhelper.Decorators.log import logging, logging_setup
from TDhelper.Decorators.performance import performance
from TDhelper.network.rpc.Core.token import token_manage
performance.on_off(True)

class client():
    __TOKEN_KEY__ = "api-token"
    __LOGGER__ = logging
    __service_exists__= lambda o,k: True if k in o.__dict__ else False
    __get_service__= lambda o,k: getattr(o,k) if o.__service_exists__(k) else None
    __token_manage:token_manage= None
    __call_time_out=2
    @performance.performance_testing
    def __init__(self, uri, path: str = "", service_conf: dict = {}, sniffer: str = "api/sniffer", services: list = [], token_key=None, logger: dict = None, try_count:int=5, try_sleep:int=3, call_time_out=2)->None:
        '''rpc client.
        
        params:
        
            uri - <class: string>: rpc gateway;
            
            path - <class: str>: rpc init url path;
            
            service_conf - <class: dict>: {"service_key":"","secret":""};
            
            sniffer - <class: str>: rpc service sniffer path, get it with operator;
            
            services - <class: list>: get rpc service by service list, default all, default all; 
            
            token_key - <class: string>: key field  for rpc authorized. it is request headers field; validate in SAAS-CLI framework core.filter.access.py;
            
            logger - <class: dict>: logger configure dict;
            
            try_count - <class:int>: max count for service available sniffer, default 5;
            
            try_sleep -<class: int>: service available sniffer retry sleep time, default 3 second;
            
            call_time_out - <class:int> remote call time out, default 2 second; 
        '''
        if logger:
            self.__LOGGER__.basicConfig(level=logging.INFO)
            self.__LOGGER__.config.dictConfig(logger)
        self.__TOKEN_KEY__ = token_key if token_key else self.__TOKEN_KEY__
        self.__call_time_out= call_time_out
        self.__token_manage= token_manage(
                token_uri= uri,
                svr_key=service_conf.get("service_key",""),
                secret=service_conf.get("secret",""),
            )
        header = {
            "access-source":"rpc"
        }
        if self.__token_manage.Status:
            try:
                header.update({self.__TOKEN_KEY__:self.__token_manage.Token})
            except Exception as e:
                raise Exception("rpc generate token error.%s"%self.__token_manage.ErrMsg)
        else:
            raise Exception(self.__token_manage.ErrMsg)
        uri = uri.rstrip("/")
        rpc_uri ="/".join([uri,path.lstrip("/")])
        if services:
            rpc_uri += "?key="+",".join(services)
        rpc_server_state = False
        for v in range(0, try_count):
            body = self.__get_conf__("/".join([uri,sniffer.lstrip("/")]), header)
            if body:
                if body['state'] == 200:
                    rpc_server_state = True
                    break
            time.sleep(try_sleep)
        if not rpc_server_state:
            raise Exception("rpc server(%s) can not access ." % uri)
        self.__context__ = self.__get_conf__(rpc_uri, header)
        if self.__context__:
            if self.__context__["state"]==200:
                if self.__context__["msg"]:
                    self.__dict__["__srv_struct__"] = {}
                    for k, v in self.__context__["msg"].items():
                        self.__dict__["__srv_struct__"][k] = type(
                            k, (RPC_SERVICE,), {}).create_by_cnf(v, self.__TOKEN_KEY__, self.__token_manage,self.__call_time_out)
                        methods = [kv for kv in v['methods'].keys()]
                        self.__dict__[k] = type(k, (dynamic_creator,), {
                                                "__dynamic_methods__": methods, "__hook_method__": self.__call__})()
                else:
                    self.__LOGGER__.error("not found rpc method.%s"%self.__context__["msg"])
                    raise Exception("not found rpc method.%s"%self.__context__["msg"])
            else:
                err_str= ",".join(["rpc service has an error: %s"%self.__context__["msg"],"access uri(%s)"%rpc_uri,"head(%s)"%json.dumps(header)])
                raise Exception(err_str)
        else:
            raise Exception("not found rpc instance config.")
        super(client, self).__init__()
    
    @performance.performance_testing
    def __call__(self, service_name, fun_name, *args, **kwargs):
        remote_func_name = ("".join([service_name, '.', fun_name]))
        if service_name in self.__srv_struct__:
            if hasattr(self.__srv_struct__[service_name], 'methods'):
                if fun_name in self.__srv_struct__[service_name].methods:
                    try:
                        start = time.clock_gettime(0)
                        result = self.__srv_struct__[service_name].__remote_call__(
                            fun_name, *args, **kwargs)
                        end = time.clock_gettime(0)
                        self.__LOGGER__.info("call %s, time consuming %f(s)" % (
                            remote_func_name, (end-start)))
                        return result
                    except Exception as e:
                        self.__LOGGER__.error(e)
                else:
                    self.__LOGGER__.error(
                        "in service(%s) not found method(%s)", (service_name, fun_name))
            else:
                self.__LOGGER__.error("service obj not found key(methods).")
        else:
            self.__LOGGER__.error('not found service(%s).' % service_name)
        return {"state": -1, "msg": {}}

    def __get_conf__(self, uri, header={}):
        state, body = GET(uri, http_headers=header,
                          time_out=5, charset="utf-8")
        if state == 200:
            body = json.loads(body)
            return body
        return {
            "state": state,
            "msg":body
        }
    
    def reload(self):
        pass
    
    def monitor(self,speed=None):
        '''
            start monitor service health.
            
            Args:
            
                speed: <cls,int>, monitor speed. seconds.
        '''
        if self.__srv_struct__:
            for o in self.__srv_struct__:
                self.__srv_struct__[o].monitor(speed)
                
    def stop_monitor(self):
        '''
            stop monitor service health.
        '''
        if self.__srv_struct__:
            for o in self.__srv_struct__:
                self.__srv_struct__[o].stop_monitor()



''' Init Applicatio
Shell command: saas --name [name] --path [path] --git [git url]; 

e.g
    sudo saas --name example_project --path ./example_project --git https://gitee.com/TonyDon/saas-cli.git

Args:
    --name: project name. to replace configrue files.
    --path: init project path. if it's null then init project by shell directory.
    --git: git url, if it's null clone by default.
    
Note
    More helpe doc. to read project README.MD
'''

import os
import sys


def CMD():
    try:
        args = buildArgs(sys.argv)
        cmd_line = "".join(["sudo git clone ", args["git"], " ", args["path"]])
        result = os.system(cmd_line)
        if result == 0:
            if args["name"]:
                result = os.system(
                    "sudo sed -i 's/{project_name}/'"+args['name']+"'/g' " + args['path']+"/manage.py")
                result = os.system(
                    "sudo sed -i 's/{project_name}/'"+args['name']+"'/g' " + args['path']+"/app_name/asgi.py")
                result = os.system(
                    "sudo sed -i 's/{project_name}/'"+args['name']+"'/g' " + args['path']+"/app_name/wsgi.py")
                result = os.system(
                    "sudo sed -i 's/{project_name}/'"+args['name']+"'/g' " + args['path']+"/make/cfg-example.ini")
                result = os.system(
                    "sudo sed -i 's/{service_name}/'"+args['name']+"'/g' " + args['path']+"/make/docker_init.ini")
                result = os.system("".join(
                    ["sudo mv ", args['path'], "/app_name ", " ", args['path'], "/", "/", args["name"]]))
                result = os.system(
                    "sudo sed -i 's/{project_name}/'"+args['name']+"'/g' " + args['path']+"/DockerFile.arm")
                result = os.system(
                    "sudo sed -i 's/{project_name}/'"+args['name']+"'/g' " + args['path']+"/DockerFile.x86")
                if result == 0:
                    result = os.system(
                        "sudo sed -i 's/{project_name}/'"+args['name']+"'/g' "+args['path']+"/"+args['name']+"/settings.py")
                    result = os.system(
                        "sudo sed -i 's/{project_name}/'"+args['name']+"'/g' " + args['path']+"/make/uwsgi.example.ini")
                    result = os.system("sudo rm -rf "+args['path']+"/.git")
                result= os.system("sudo cp -r"+args['path']+"/make/cfg-example.ini"+args["path"]+"/config/development.ini")
                result= os.system("sudo cp -r"+args['path']+"/make/cfg-example.ini"+args["path"]+"/config/test.ini")
        else:
            print('git clone error.')
    except Exception as e:
        print(e)


def buildArgs(args):
    result = {
        "name": "project_name",
        "path": os.path.abspath(os.curdir),
        "git": "https://gitee.com/TonyDon/saas-cli.git"
    }
    if args:
        o = 0
        while True:
            if o+1 >= len(args):
                break
            lower_cmd = args[o].lower()
            if lower_cmd == "--name" or lower_cmd == "--n":
                o += 1
                result["name"] = args[o]
            elif lower_cmd.lower() == "--path" or lower_cmd == "--p":
                o += 1
                result["path"] = args[o]
            elif lower_cmd.lower() == "--git" or lower_cmd == "--g":
                o += 1
                result["git"] = args[o]
            o += 1
    return result


if __name__ == '__main__':
    CMD()


#!/usr/bin/env python3.6
# -*- encoding: utf-8 -*-
"""
@File    :   transformationType.py
@Time    :   2020/07/30 23:34:47
@Author  :   Tang Jing 
@Version :   1.0.0
@Contact :   yeihizhi@163.com
@License :   (C)Copyright 2020
@Desc    :   类型转换
"""

# here put the import lib
import datetime
from pickle import TRUE

# code start


def transformation(value: any, transformationType, split_str=" ", default_None=""):
    """
    类型转换(目前仅只支持int,str,datetime,bool转换)
    - Params:
        - value: 需要转换的内容.
        - transformationType: 转换后的类型.
        - split_str: 字符转数组分割字符，默认一个空格, 如字符串内不包含分割字符则会直接把字符串按每个字符分割；
    """
    if not value:
        return default_None
    try:
        if not isinstance(value, transformationType):
            if transformationType == int:
                value = int(value)
            if transformationType == str:
                if isinstance(value, datetime.datetime):
                    # 日期类型转字符
                    value = datetime.datetime.strftime(value, "%Y-%m-%d %H:%M:%S")
                elif isinstance(value, datetime.date):
                    value = datetime.date.strftime(value, "%Y-%m-%d")
                else:
                    value = str(value)
            if transformationType == float:
                try:
                    if isinstance(
                        value,
                        (
                            str,
                            int,
                        ),
                    ):
                        value = float(value)
                    else:
                        raise Exception(
                            "transformation float, value type must is str. value: %s"
                            % value
                        )
                except Exception as e:
                    raise e
            if transformationType == datetime.datetime:
                if isinstance(value, str):
                    value = value.replace("/", "-")
                    if len(value) > 10:
                        value = datetime.datetime.strptime(value, "%Y-%m-%d %H:%M:%S")
                    else:
                        value = datetime.datetime.strptime(value, "%Y-%m-%d")
                else:
                    raise Exception(
                        "transformation datetime, value type must is str. value:%s"
                        % value
                    )
            if transformationType == datetime.date:
                if isinstance(value, str):
                    value = value.replace("/", "-")
                    if len(value) > 10:
                        value = datetime.datetime.strptime(
                            value, "%Y-%m-%d %H:%M:%S"
                        ).date()
                    else:
                        value = datetime.datetime.strptime(value, "%Y-%m-%d").date()
                else:
                    raise Exception(
                        "transformation date, value type must is str. value: %s" % value
                    )
            if transformationType == bool:
                if isinstance(value, str):
                    v = value.lower()
                    if not v:
                        value = False
                    elif v == "false":
                        value = False
                    elif v == "true":
                        value = True
                    else:
                        value = True
                else:
                    value = bool(value)
            if transformationType == list:
                if isinstance(value, str):
                    if value.__contains__(split_str):
                        value= value.split(split_str)
                    else:
                        value= list(value)
    except Exception as e:
        raise e
    return value



#! python 3.6

#########################################################
# | @File    :   protocol.py
# | @ClsName :   TDhelper.network.websocket.protocol.webProtocol
# | @Version :   1.0.0
# | @History :
# |-----------------------------------------------------------------|
# | Type   | Author   | Contact           | Time                    |
# |-----------------------------------------------------------------|
# | Create | Tony.don | yeihizhi@163.com  | 2021-10-30 23:23:02     |
# |-----------------------------------------------------------------|
# | @License :   MIT
# | @Desc    :   webosocket action protocol.
#########################################################

# import lib by pypi
import abc
import asyncio
import json

# import lib by project


class webProtocol(metaclass=abc.ABCMeta):
    def __init__(self, authorize=False):
        """__init__

        webProtocol abstract class.

        Args:
            None.

        Returns:
            None.

        Raises:
            None.

        """
        self.isAuthorize = authorize
        self.usr_list = {}

    async def authorize(self, websocket, token):
        """authorize

        user connect authorize. if authorize success then insert this client handle in usr_list.

        Args:
            websocket (websocket): client websocket handle.

        Returns:
            True: authorize success.
            Flase: authorize fail.

        Raises:
            None.

        """
        self.usr_list = {token: websocket}
        # todo:  authorize.
        return False

    async def loginout(self, websocket, token):
        '''loginout
        
        user login out server.
        
        Args:
            websocket (websocket): client handle.
            token (string): user token.
        
        Returns:
            None.
        
        Raises:
            None.
        
        '''
        if self.usr_list[token]:
            self.usr_list[token] = None

    async def disableconnection(self, websocket, token=None):
        '''disableconnection
        
        disable an connection websocket.
        
        Args:
            websocket (websocket): client handle.
            token (object): user token.
        
        Returns:
            None.
        
        Raises:
            None.
        
        '''
        if not token:
            await self.loginout(websocket, token)
        try:
            await websocket.wait_close()
        except Exception as e:
            pass

    async def ping(self, websocket, data=None):
        try:
            pong_waiter = await websocket.ping(data)
            await pong_waiter
        except Exception as e:
            await self.disableconnection(websocket)

    @abc.abstractclassmethod
    async def protocolAnalysis(self, websocket, msg):
        '''protocolAnalysis
        
        Analysis action protocol command, router action method.
        
        Args:
            websocket (websocket): client handle.
            msg (object): server recv client msg. 
        
        Returns:
            None.
        
        Raises:
            None.
        
        '''
        pass

    @abc.abstractclassmethod
    async def response(self, websocket):
        '''response
        
        response msg to client.
        
        Args:
            websocket (websocket): client handle.
        
        Returns:
            None.
        
        Raises:
            None.
        
        '''
        pass



from types import FunctionType
from TDhelper.db.mongodb.orm.core.field_type import *
class field:
    model_type:field_type= None
    default= None
    max_length= None
    validate_funs:list|FunctionType= None
    is_none= False
    value= None
    
    def __init__(self,type:field_type, default_value=None,max_length=None,none:bool=False,valid_list:list|FunctionType= None) -> None:
        self.model_type=type
        self.default= default_value
        self.max_length= max_length
        self.is_none= none
        self.validate_funs= valid_list
        if type == field_type.String:
            if not max_length:
                raise ValueError("string field must set max length.")


#!/usr/bin/env python
# -*- coding:utf-8 -*-

import setuptools

with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()
setuptools.setup(
    name="TDhelper",
    version="2.8.6",
    keywords=("pip", "TDhelper", "featureextraction"),
    description="Helper cls.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="Apache 2.0",
    url="https://gitee.com/TonyDon/pyLib",
    author="TangJing",
    author_email="yeihizhi@163.com",
    packages=setuptools.find_packages(exclude=["UnitTest"]),
    classifiers=[],
    install_requires=[
        "crypto==1.4.1",
        "openpyxl==3.1.2",
        "pymongo==4.3.3",
        "requests==2.28.2",
        "six==1.16.0",
        "urllib3==1.26.15",
        "mysql-connector>=2.2.9"
],
entry_points = {
        'console_scripts': [
            #'foo = demo:test',
            #'bar = demo:test',
            'saas = TDhelper.shellScripts.saasHelper:CMD'
        ],
        'gui_scripts': [
            #'baz = demo:test',
        ]
    }
)
